/*
 OGG Vorbis and MP3 player
 Copyright (C) 2002 Clinton Ebadi
 Covered under the GPL Version 2 or (at your option) any later version
 THIS IS _FREE_ SOFTWARE. Free with a capital 'F'! None of that 
 watered down Open Source! Free Software now! www.fsf.org


 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Or grab a copy at http://unknownlamer.org/GPLv2 (if it still exists)
*/

#include <unistd.h>
#include <string.h>

static int use_q = 0; /* if 1, then pass quiet to player */

static void decode (const char *song);
static void ogg_decode (const char *song);
static void mp3_decode (const char *song);

int main (int argc, char **argv)
{
  if (argc < 2)
    return 2; /* no execution */
  if (strcmp (argv[1], "-q") == 0)
    {
      use_q = 1;
      decode (argv[2]);
    }
  else
    decode (argv[1]);

  return 0; /* we won't get here */
}

static void decode (const char *song)
{
  char *extension;

  if (song == NULL)
    return;

  extension = strrchr (song, '.');
  if (extension == NULL)
    return;
  ++extension; /* kill '.' */

  if (strcmp (extension, "ogg") == 0)
    ogg_decode (song);
  else if (strcmp (extension, "mp3") == 0)
    mp3_decode (song);

  /* do nothing for unsupported formats */
  return;
}

void ogg_decode (const char *song)
{
  char *ogg_argv[4] = 
    { "ogg123", song, NULL, NULL };

  if (use_q)
    ogg_argv[2] = "-q";
  
  execvp ("ogg123", ogg_argv);
}

void mp3_decode (const char *song)
{
  char *mp3_argv[4] = 
    { "mpg321", song, NULL, NULL };

  if (use_q)
    mp3_argv[2] = "-q";
  
  execvp ("mpg321", mp3_argv);

}
