/* osd_display -- based on the osd_cat example program */
/* Copyright (C) 2002 Clinton Ebadi */
/* there were no messages here before */
/*  
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Or grab a copy at http://lamer.hackedtobits.com/GPLv2 (if it still exists)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <xosd.h>
#include <unistd.h>
#include <getopt.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/un.h>

static struct option long_options[] = {
    {"font",   1, NULL, 'f'},
    {"color",  1, NULL, 'c'},
    {"delay",  1, NULL, 'd'},
    {"offset", 1, NULL, 'o'},
    {"top",    0, NULL, 't'},
    {"bottom", 0, NULL, 'b'},
    {"help",   0, NULL, 'h'},
    {NULL,     0, NULL, 0}
};

static int osd_disp_opensocket ();
static void osd_disp_closesocket (int sock);

static void osd_disp_sighandle (int sig);

int main (int argc, char *argv[])
{
  int server_sock, client_sock;
  int client_len;
  struct sockaddr_un client_addr;
  xosd *osd;
  char buffer [1024];
  char *newline;
  int read_size = 1025;
  
  char *font = "fixed";
  char *color = "red";
  int delay = 5;
  xosd_pos pos = XOSD_bottom;
  int offset = 10;
  
  while (1)
    {
      int option_index = 0;
      int c = getopt_long (argc, argv, "f:c:d:o:tbh", long_options, &option_index);
      if (c == -1) break;
      switch (c)
	{
	case 'f':
	  font = optarg;
	  break;
	case 'c':
	  color = optarg;
	  break;
	case 'd':
	  delay = atoi(optarg);
	  break;
	case 'o':
	  offset = atoi(optarg);
	  break;
	case 't':
	  pos = XOSD_top;
	  break;
	case 'b':
	  pos = XOSD_bottom;
	  break;
	case '?':
	case 'h':
	  fprintf (stderr, "Usage: %s [OPTION] ...\n", argv[0]);
	  fprintf (stderr, "Display FILE, or standard input, on top of display.\n\n");
	  fprintf (stderr, "  -t, --top           Display at top of screen\n");
	  fprintf (stderr, "  -b, --bottom        Display at bottom of screen\n");
	  fprintf (stderr, "  -f, --font=FONT     Use font\n");
	  fprintf (stderr, "  -c, --color=COLOR   Use color\n");
	  fprintf (stderr, "  -d, --delay=TIME    Show for specified time\n");
	  fprintf (stderr, "  -o, --offset=OFFSET Display Offset\n");
	  fprintf (stderr, "  -h, --help          Show this help\n");
	  fprintf (stderr, "\nData is read from, /tmp/%s-osd.\n", 
		   getenv ("USER"));
	  return EXIT_SUCCESS;
	}
    }
 
  signal (SIGINT, osd_disp_sighandle);
  signal (SIGTERM, osd_disp_sighandle);
  server_sock = osd_disp_opensocket ();
  
  osd = xosd_init (font, color, delay, pos, offset, 2);
  if (!osd)
    {
      fprintf (stderr, "Error initializing osd\n");
      return EXIT_FAILURE;
    }
  
  listen (server_sock, 5);
  while (1)
    {
      client_len = sizeof (client_addr);
      client_sock = accept (server_sock, &client_addr, &client_len);
      read_size = read (client_sock, &buffer, 1024);
      if ((newline = strchr (buffer, '\n')))
	newline[0] = '\0';
      xosd_display (osd, 0, XOSD_string, buffer);
      close (client_sock);
    }
  
  sleep(delay);
  
  xosd_uninit (osd);
  osd_disp_closesocket (server_sock);
  return EXIT_SUCCESS;
}

int osd_disp_opensocket ()
{
  char buffer[1024];
  int server_sock;
  int server_len;
  struct sockaddr_un server_addr;
  snprintf (buffer, sizeof (buffer), "/tmp/%s-osd", getenv ("USER"));
  unlink (buffer);
  server_sock = socket (AF_UNIX, SOCK_STREAM, 0);

  server_addr.sun_family = AF_UNIX;
  strcpy (server_addr.sun_path, buffer);
  server_len = sizeof (server_addr);
  bind (server_sock, &server_addr, server_len);

  return server_sock;
}

void osd_disp_closesocket (int sock)
{
  char buffer[1024];
  close (sock);
  snprintf (buffer, sizeof (buffer), "/tmp/%s-osd", getenv ("USER"));
  unlink (buffer);
}

void osd_disp_sighandle (int sig)
{
  /* remove /tmp/$USER-osd ... */
  /* this could be coded much more nicely */
  char buffer[2048] = "";  
  char *user = getenv ("USER");
  strncat (buffer, "/bin/rm -f /tmp/", 2047);
  strncat (buffer, user, 2047);
  strncat (buffer, "-osd", 2047);

  system (buffer);
  exit (0);
}
