/*  osd_support.c (On Screen Display support) -- part of wmsndplay
    Copyright (C) 2002 Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "osd_support.h"
#include <xosd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <signal.h>

static pid_t pid_of_osd;
int xosd_on = 1; /* defaults to on */

/* we want this no matter what */
void set_xosd_on (char *s)
{
    char *temp = s;
    while (*temp)
      {
	*temp = tolower(*temp);
	temp++;
      }
    if ( !strcmp(s, "on") || !strcmp(s, "yes") || !strcmp(s, "1") ||
	 !strcmp(s, "true"))
      {
	xosd_on = 1;
      } else
	xosd_on = 0;
}

 #if (HAVE_LIBXOSD == 1)
//#if 0 /* keep osd disabled for now */

#define font "-adobe-helvetica-medium-r-normal-*-*-180-*-*-p-*-iso10646-1"

static void wms_drawstring (const char* str)
{
  int sockfd;
  int len;
  struct sockaddr_un address;
  int result;
  char ch[1024];
  char addr[1024];

  snprintf (addr, 1024, "/tmp/%s-osd", getenv ("USER"));

  sockfd = socket (AF_UNIX, SOCK_STREAM, 0);
  address.sun_family = AF_UNIX;
  strcpy (address.sun_path, addr);
  len = sizeof (address);

  result = connect (sockfd, &address, len);

  if (result == -1)
    perror ("something wierd happened in client");
   
  sprintf (&ch, str);
  write (sockfd, &ch, 1024);
  close (sockfd);
}

void wms_init ()
{
  static int init = 0;
  if (!init)
    {
      pid_of_osd = fork ();
      if (!pid_of_osd)
	execlp ("osd_display", "osd_display", "--font", font, NULL);
      init = 1;
      sleep (1); /* give osd_display a sec to start */
    }
  if (xosd_on)
    wms_drawstring ("wmsndplay");
}

void wms_displaydir (const char* dirname)
{
  if (!xosd_on)
    return;
  wms_drawstring (dirname);
  return;
}

void wms_displaytitle (const char* title)
{
  if (!xosd_on)
    return;
  wms_drawstring (title);
  return;
}

void wms_exit ()
{
  kill (SIGKILL, pid_of_osd);
}

#else /* if HAVE_LIBXOSD == 0 */
/* just make the functions null ops if osd is disabled */
void wms_displaydir (const char* dirname) { }

void wms_displaytitle (const char* title) { }

void wms_init () { }
void wms_exit () { }

#endif /* HAVE_LIBXOSD */
